/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2014        */
/*-----------------------------------------------------------------------*/
/* If a working storage control module is available, it should be        */
/* attached to the FatFs via a glue function rather than modifying it.   */
/* This is an example of glue functions to attach various exsisting      */
/* storage control modules to the FatFs module with a defined API.       */
/*-----------------------------------------------------------------------*/
#include "diskio.h"			/* FatFs lower layer API */
#include "at32_sdio.h"
#include "w25qxx.h"
#include "malloc.h"	

#define SD_CARD	 0  //SD,Ϊ0
#define EX_FLASH 1	//ⲿflash,Ϊ1

#define FLASH_SECTOR_SIZE 	512			  
//W25Q128
//ǰ12Mֽڸfatfs,12Mֽں,ڴֿ,ֿռ3.09M.	ʣಿ,ͻԼ	 			    
u16	    FLASH_SECTOR_COUNT=2048*12;	//W25Q1218,ǰ12MֽڸFATFSռ
#define FLASH_BLOCK_SIZE   	8     	//ÿBLOCK8


//ô״̬
DSTATUS disk_status (
	BYTE pdrv		/* Physical drive nmuber to identify the drive */
)
{ 
	return RES_OK;
}  
//ʼ
DSTATUS disk_initialize (
	BYTE pdrv				/* Physical drive nmuber to identify the drive */
)
{
	u8 res=0;	    
	switch(pdrv)
	{
		case SD_CARD://SD
			res=sd_init();//SDʼ 
  			break;
		case EX_FLASH://ⲿflash
			W25QXX_Init();
			FLASH_SECTOR_COUNT=2048*12;//W25Q1218,ǰ12MֽڸFATFSռ 
 			break;
		default:
			res=1; 
	}		 
	if(res)return  STA_NOINIT;
	else return 0; //ʼɹ 
} 
//
//pdrv:̱0~9
//*buff:ݽջ׵ַ
//sector:ַ
//count:Ҫȡ
DRESULT disk_read (
	BYTE pdrv,		/* Physical drive nmuber to identify the drive */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address in LBA */
	UINT count		/* Number of sectors to read */
)
{
	u8 res=0; 
    if (!count)return RES_PARERR;//countܵ0򷵻ز		 	 
	switch(pdrv)
	{
		case SD_CARD://SD
			res=sd_disk_read(buff,sector,count);	 
			while(res)//
			{
				sd_init();	//³ʼSD
				res=sd_disk_read(buff,sector,count);	
				//printf("sd rd error:%d\r\n",res);
			}
			break;
		case EX_FLASH://ⲿflash
			for(;count>0;count--)
			{
				W25QXX_Read(buff,sector*FLASH_SECTOR_SIZE,FLASH_SECTOR_SIZE);
				sector++;
				buff+=FLASH_SECTOR_SIZE;
			}
			res=0;
			break;
		default:
			res=1; 
	}
   //ֵSPI_SD_driver.cķֵתff.cķֵ
    if(res==0x00)return RES_OK;	 
    else return RES_ERROR;	   
}
//д
//pdrv:̱0~9
//*buff:׵ַ
//sector:ַ
//count:Ҫд
#if _USE_WRITE
DRESULT disk_write (
	BYTE pdrv,			/* Physical drive nmuber to identify the drive */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address in LBA */
	UINT count			/* Number of sectors to write */
)
{
	u8 res=0;  
    if (!count)return RES_PARERR;//countܵ0򷵻ز		 	 
	switch(pdrv)
	{
		case SD_CARD://SD
			res=sd_disk_write((u8*)buff,sector,count);
			while(res)//д
			{
				sd_init();	//³ʼSD
				res=sd_disk_write((u8*)buff,sector,count);	
				//printf("sd wr error:%d\r\n",res);
			}
			break;
		case EX_FLASH://ⲿflash
			for(;count>0;count--)
			{										    
				W25QXX_Write((u8*)buff,sector*FLASH_SECTOR_SIZE,FLASH_SECTOR_SIZE);
				sector++;
				buff+=FLASH_SECTOR_SIZE;
			}
			res=0;
			break;
		default:
			res=1; 
	}
    //ֵSPI_SD_driver.cķֵתff.cķֵ
    if(res == 0x00)return RES_OK;	 
    else return RES_ERROR;	
}
#endif
//Ļ
//pdrv:̱0~9
//ctrl:ƴ
//*buff:/ջָ
#if _USE_IOCTL
DRESULT disk_ioctl (
	BYTE pdrv,		/* Physical drive nmuber (0..) */
	BYTE cmd,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
DRESULT res;						  			     
	if(pdrv==SD_CARD)//SD
	{
	    switch(cmd)
	    {
		    case CTRL_SYNC:
				res = RES_OK; 
		        break;	 
		    case GET_SECTOR_SIZE:
				*(DWORD*)buff = 512; 
		        res = RES_OK;
		        break;	 
		    case GET_BLOCK_SIZE:
				*(WORD*)buff = sd_card_info.card_blk_size;
		        res = RES_OK;
		        break;	 
		    case GET_SECTOR_COUNT:
		        *(DWORD*)buff = sd_card_info.card_capacity/512;
		        res = RES_OK;
		        break;
		    default:
		        res = RES_PARERR;
		        break;
	    }
	}else if(pdrv==EX_FLASH)	//ⲿFLASH  
	{
	    switch(cmd)
	    {
		    case CTRL_SYNC:
				res = RES_OK; 
		        break;	 
		    case GET_SECTOR_SIZE:
		        *(WORD*)buff = FLASH_SECTOR_SIZE;
		        res = RES_OK;
		        break;	 
		    case GET_BLOCK_SIZE:
		        *(WORD*)buff = FLASH_BLOCK_SIZE;
		        res = RES_OK;
		        break;	 
		    case GET_SECTOR_COUNT:
		        *(DWORD*)buff = FLASH_SECTOR_COUNT;
		        res = RES_OK;
		        break;
		    default:
		        res = RES_PARERR;
		        break;
	    }
	}else res=RES_ERROR;//Ĳ֧
    return res;
}
#endif
//ʱ
//User defined function to give a current time to fatfs module      */
//31-25: Year(0-127 org.1980), 24-21: Month(1-12), 20-16: Day(1-31) */                                                                                                                                                                                                                                          
//15-11: Hour(0-23), 10-5: Minute(0-59), 4-0: Second(0-29 *2) */                                                                                                                                                                                                                                                
DWORD get_fattime (void)
{				 
	return 0;
}			 
//̬ڴ
void *ff_memalloc (UINT size)			
{
	return (void*)mymalloc(SRAMIN,size);
}
//ͷڴ
void ff_memfree (void* mf)		 
{
	myfree(SRAMIN,mf);
}

















