/*
 **************************************************************************
 * File Name    : wm8988_i2c.c
 * Description  : wm 8988  driver
 * Author       : Artery SW Team
 * Date         : 2018-11-26
 * Version      : V1.0.0
 **************************************************************************
 */
 
#include "at32f403a_407.h"
#include "wm8988_i2s.h"
#include "at32_audio_conf.h"

/*I2S1 Pin*/ 
#define I2S1_WS_PIN                GPIO_PINS_4
#define I2S1_CK_PIN                GPIO_PINS_5
#define I2S1_SD_PIN                GPIO_PINS_7
#define I2S1_GPIO                  GPIOA
#define I2S1_GPIO_RCC_CLK          CRM_GPIOA_PERIPH_CLOCK
#define I2S1_DR_ADDRESS            0x4001300C

/*I2S2 Pin*/
//#define I2S2_WS_PIN                GPIO_Pins_12
//#define I2S2_CK_PIN                GPIO_Pins_13
//#define I2S2_SD_PIN                GPIO_Pins_15
//#define I2S2_GPIO                  GPIOB
//#define I2S2_GPIO_RCC_CLK          RCC_APB2PERIPH_GPIOB
//#define I2S2_DR_ADDRESS            0x4000380C


#define I2S_FREQ_16                16000
#define I2S_FREQ_48                48000
#define I2S_FREQ_44                44100

#define I2S_DATA_FIFOBUFFER_MAX    192 * 20 + 192
#define I2S_DATA_VALIDBUFFER_MAX  (I2S_DATA_FIFOBUFFER_MAX - 192) / 2
#define I2S_WRITE_LEN_ONTIMES      1
typedef struct _I2S_Fifo_Buffer
{
    uint16_t u16I2S1_SpeakerFiFoBuffer[I2S_DATA_FIFOBUFFER_MAX]; /*Speaker Fifo buffer mose*/
    uint16_t u16I2S1_WritePointer;
    uint16_t u16I2S1_ReadPointer;
    uint32_t u32NbyteWr;
}I2S_Fifo_Buffer;

I2S_Fifo_Buffer I2S_FiFoSpeaker = {{0}, 0, 0, 0};
I2S_TransBuffer I2S1_SpeakerBuffer = {{0}, {0}, 0, 0};                /* Speaker use buffer*/
//I2S_TransBuffer I2S2_MicrphoneBuffer = {{0}, {0}, 0, 0};;                /* Micrphone use buffer*/
short * /*volatile*/ P_AudioBuff;

typedef struct __I2S_Config__
{
    uint32_t u32SpeakerBufLen;    /* Speaker Send data from I2S to WM8988 1 frame length*/  
    uint32_t u32SpeakerBufLenB;    /* Speaker Send data from I2S to WM8988 1 frame length*/
    uint32_t u32MicrphoneBufLen;    /* Micrphone Get data from WM8988 to I2S 1 frame length*/
    uint32_t u32I2SBitw;
    uint32_t u32I2SFreq;
    
}I2S_Config;

I2S_Config i2s_conf = {64, 16, 44100};  /*Tx Buffer len default is 64
                                         * bitw default is 16 bit
                                         * Freq default is 16000
                                        */

//static uint8_t *u8I2SCurBuffer = 0;
i2s_init_type I2S_InitStructure;
dma_init_type DMA_InitStructure;
uint8_t u8ZeroBuffer[I2S_MAX_TXLEN] = {0};

 /**
  * @brief  I2S initialize
  *         GPIO, I2S, DMA, Interrut initialize
  * @param  None
  * @retval None
  */
void AT32_I2S_Init(uint32_t freq, uint32_t bitw)
{
  gpio_init_type GPIO_InitStructure;  
  /*init gpio struct*/ 
  i2s_conf.u32I2SFreq = freq;
  i2s_conf.u32I2SBitw = bitw;
  i2s_conf.u32SpeakerBufLen = (i2s_conf.u32I2SFreq / 1000) * (i2s_conf.u32I2SBitw / 8) * 2;     /*Stereo*/
  //    i2s_conf.u32MicrphoneBufLen = (i2s_conf.u32I2SFreq / 1000) * (i2s_conf.u32I2SBitw / 8) * 2;     /*Stereo*/ 
  I2S1_SpeakerBuffer.u8I2SCurBuffer = 0;
  //    I2S2_MicrphoneBuffer.u8I2SCurBuffer = 0; 

  /*********************************I2S1 initialize Speaker******************************/
  /*I2S1 is Slave Tx, Send data to WM8988*/

  /*Enable I2S1 GPIO Clock*/
  crm_periph_clock_enable(I2S1_GPIO_RCC_CLK, TRUE); 

  /*PA4-->I2S1_WS, PA5-->I2S1_CK,PA7-->I2S1_SD*/
  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = I2S1_WS_PIN | I2S1_CK_PIN;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_NONE;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(I2S1_GPIO, &GPIO_InitStructure);

  GPIO_InitStructure.gpio_pins = I2S1_SD_PIN;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_init(I2S1_GPIO, &GPIO_InitStructure);

  /*Enable I2S1 Clock*/
  crm_periph_clock_enable(CRM_SPI1_PERIPH_CLOCK, TRUE);
  /*Reset I2S1*/
  spi_i2s_reset(SPI1); 
  /*initialize I2S Struct*/
  i2s_default_para_init(&I2S_InitStructure);
  if ( i2s_conf.u32I2SFreq == I2S_FREQ_16 ) /*freq is 48K*/
    I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_16K;
  else if ( i2s_conf.u32I2SFreq == I2S_FREQ_48 )
    I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_48K; 
  else if ( i2s_conf.u32I2SFreq == I2S_FREQ_44 )
    I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_44_1K; 


  I2S_InitStructure.audio_protocol = I2S_AUDIO_PROTOCOL_PHILLIPS;
  I2S_InitStructure.clock_polarity = I2S_CLOCK_POLARITY_LOW;
  if ( i2s_conf.u32I2SBitw == 24 )
    I2S_InitStructure.data_channel_format = I2S_DATA_24BIT_CHANNEL_32BIT;  /*24 bit wide*/
  else
    I2S_InitStructure.data_channel_format = I2S_DATA_16BIT_CHANNEL_16BIT;

  I2S_InitStructure.mclk_output_enable = FALSE;
  I2S_InitStructure.operation_mode = I2S_MODE_SLAVE_TX;
  i2s_init(SPI1, &I2S_InitStructure);
  /* Enable I2S1 DMA request source */
  spi_i2s_dma_transmitter_enable(SPI1,TRUE);
  i2s_enable(SPI1, TRUE);

  /*I2S1 DMA Configure, I2S1 TX DMA map DMA1_Channel3*/
  /*reset DMA1_Channel3*/
  crm_periph_clock_enable(CRM_DMA1_PERIPH_CLOCK, TRUE);
  dma_reset(DMA1_CHANNEL3);
  DMA_InitStructure.buffer_size = OUTPUT_BUFFER_SIZE;
  DMA_InitStructure.direction = DMA_DIR_MEMORY_TO_PERIPHERAL;
  DMA_InitStructure.memory_base_addr = (uint32_t)P_AudioBuff;
  DMA_InitStructure.memory_data_width = DMA_MEMORY_DATA_WIDTH_HALFWORD;
  DMA_InitStructure.memory_inc_enable = TRUE;
  DMA_InitStructure.loop_mode_enable = FALSE;
  DMA_InitStructure.peripheral_base_addr = I2S1_DR_ADDRESS;
  DMA_InitStructure.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_HALFWORD;
  DMA_InitStructure.peripheral_inc_enable = FALSE;
  DMA_InitStructure.priority = DMA_PRIORITY_HIGH;
  dma_init(DMA1_CHANNEL3, &DMA_InitStructure);
  dma_interrupt_enable(DMA1_CHANNEL3, DMA_FDT_INT, TRUE);
  dma_channel_enable(DMA1_CHANNEL3, TRUE);

  nvic_irq_enable(DMA1_Channel3_IRQn, 0, 0);  
  /*********************************End I2S1 initialize******************************/
 }
/**
  * @brief  I2S set freq 
  * @param  freq set new frequency
  * @retval None
  */
void AT32_SetI2S_FREQ(uint32_t freq)
{  
    /*Disable I2S and DMA*/
    i2s_enable(SPI1, FALSE);
    dma_channel_enable(DMA1_CHANNEL3, FALSE);
    
    i2s_enable(SPI2, FALSE);
    dma_channel_enable(DMA1_CHANNEL4, FALSE);
   
    i2s_conf.u32I2SFreq = freq;
    i2s_conf.u32SpeakerBufLen = (i2s_conf.u32I2SFreq / 1000) * (i2s_conf.u32I2SBitw / 8) * 2;     /*Stereo*/
    i2s_conf.u32MicrphoneBufLen = (i2s_conf.u32I2SFreq / 1000) * (i2s_conf.u32I2SBitw / 8) * 2;     /*Stereo*/
    /*Reset SPI*/
    spi_i2s_reset(SPI1);
    spi_i2s_reset(SPI2);
    if ( freq == I2S_FREQ_16 )
    {
        I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_16K;
    }
    else if ( freq == I2S_FREQ_48 )
    {
        I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_48K;
    }
    else if ( freq == I2S_FREQ_44 )
    {
        I2S_InitStructure.audio_sampling_freq = I2S_AUDIO_FREQUENCY_44_1K;
    }
    I2S_InitStructure.operation_mode = I2S_MODE_SLAVE_TX;
    i2s_init(SPI1, &I2S_InitStructure);
    /* Enable I2S2 DMA request source */
    spi_i2s_dma_transmitter_enable(SPI1,TRUE);
    i2s_enable(SPI1, TRUE);
    
    I2S_InitStructure.operation_mode = I2S_MODE_SLAVE_RX;
    i2s_init(SPI2, &I2S_InitStructure);
    /* Enable I2S2 DMA request source */
    spi_i2s_dma_receiver_enable(SPI2,TRUE);
    i2s_enable(SPI2, TRUE);

    
    /*Reset DMA*/
  dma_reset(DMA1_CHANNEL3);
  DMA_InitStructure.buffer_size = OUTPUT_BUFFER_SIZE;
  DMA_InitStructure.direction = DMA_DIR_MEMORY_TO_PERIPHERAL;
  DMA_InitStructure.memory_base_addr = (uint32_t)P_AudioBuff;
  DMA_InitStructure.memory_data_width = DMA_MEMORY_DATA_WIDTH_HALFWORD;
  DMA_InitStructure.memory_inc_enable = TRUE;
  DMA_InitStructure.loop_mode_enable = FALSE;
  DMA_InitStructure.peripheral_base_addr = I2S1_DR_ADDRESS;
  DMA_InitStructure.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_HALFWORD;
  DMA_InitStructure.peripheral_inc_enable = FALSE;
  DMA_InitStructure.priority = DMA_PRIORITY_HIGH;
  dma_init(DMA1_CHANNEL3, &DMA_InitStructure);
  dma_interrupt_enable(DMA1_CHANNEL3, DMA_FDT_INT, TRUE);
  dma_channel_enable(DMA1_CHANNEL3, TRUE);
}

/**
  * @brief  get current empty buffer pointer
  * @param  None
  * @retval buffer pointer
  */
uint8_t *AT32_I2S1_GetSpeakerBuffer()
{
//    return I2S1_SpeakerBuffer.u8I2SCurBuffer;
    return (uint8_t *)I2S_FiFoSpeaker.u16I2S1_SpeakerFiFoBuffer;
}


/**
  * @brief  get current buffer
  * @param  None
  * @retval buffer pointer
  */
uint8_t *AT32_I2S1_GetMicrphoneBuffer()
{
//    return I2S2_MicrphoneBuffer.u8I2SCurBuffer;
  return 0;
}


/**
  * @brief  Get Micrphone buffer length
  * @param  None
  * @retval Buffer Len
  */
uint32_t AT32_I2S1_GetMicrphoneFrameLen()
{
    return i2s_conf.u32MicrphoneBufLen;
}

/**
  * @brief  set Speaker buffer length
  * @param  buffer_len
  * @retval Buffer Len
  */
uint32_t AT32_I2S1_SetSpeakerFrameLen(uint32_t u32Len)
{
  return 0;    
}

/**
  * @brief  Write data to Speaker buffer
  * @param  u8Pointer  buffer poinger
  * @param  u16Len     Buffer len
  * @retval status
  */
uint8_t AT32_I2S1_WriteDataToSpkFifoBuffer(uint8_t *u8Pointer, uint16_t u16Len)
{
    uint16_t *u16bufer = (uint16_t *)u8Pointer;
    uint16_t *u16Fifo = I2S_FiFoSpeaker.u16I2S1_SpeakerFiFoBuffer + I2S_FiFoSpeaker.u16I2S1_WritePointer;
    uint16_t u16BufferLen = u16Len >> 1, i;
  
    for ( i = 0; i < u16BufferLen; i ++ )
    {
        if ( I2S_FiFoSpeaker.u16I2S1_WritePointer >= I2S_DATA_VALIDBUFFER_MAX )
        {
            I2S_FiFoSpeaker.u16I2S1_WritePointer = 0;
            u16Fifo = I2S_FiFoSpeaker.u16I2S1_SpeakerFiFoBuffer;
        }
        *u16Fifo ++ = *u16bufer ++;
        I2S_FiFoSpeaker.u16I2S1_WritePointer ++;

    }
    return 0;
}


/**
  * @brief  Write data to Speaker buffer
  * @param  u8Pointer  buffer poinger
  * @param  u16Len     Buffer len
  * @retval status
  */
uint32_t AT32_I2S1_GetNbyteDataSend()
{
    uint32_t u32Nbyte = I2S_FiFoSpeaker.u32NbyteWr;
    I2S_FiFoSpeaker.u32NbyteWr = 0;
    return u32Nbyte;
}
  /**
  * @brief  I2S1 TX DMA1_Channel3 IRQ Hander 
  * @param  None
  * @retval None
  */
extern u32 frame_count2;
vu16 curr_buff_done; 

void DMA1_Channel3_IRQHandler()
{
    if (dma_flag_get(DMA1_FDT3_FLAG) == SET)
    {
        dma_flag_clear(DMA1_FDT3_FLAG);
        dma_channel_enable(DMA1_CHANNEL3,FALSE);
        curr_buff_done = 1;
        frame_count2++;
        SPI1->i2sctrl &= ((uint16_t)0xFBFF);//I2S_Cmd(SPI2, DISABLE);
        DMA1_CHANNEL3->ctrl_bit.fdtien = FALSE;//DMA_ITConfig(DMA1_Channel5, DMA_IT_TC, DISABLE);
        DMA1_CHANNEL3->ctrl_bit.chen = FALSE;//DMA_Cmd(DMA1_Channel5, DISABLE);        
    }
}

#if 0
  /**
  * @brief  I2S2 RX DMA1_Channel4 IRQ Hander 
  * @param  None
  * @retval None
  */
void DMA1_Channel4_IRQHandler()
{
    if (DMA_GetITStatus(DMA1_INT_TC4) == SET)
    {
        DMA_ClearITPendingBit(DMA1_INT_TC4);
        DMA_ChannelEnable(DMA1_Channel4,DISABLE);
        if ( I2S2_MicrphoneBuffer.u8I2SBufAB == 0 )
        { /*BufferA transmit complete*/
            I2S2_MicrphoneBuffer.u8I2SBufAB = 1;        /*Next DMA use Buffer B*/
            DMA1_Channel4->CMBA = (uint32_t)I2S2_MicrphoneBuffer.u8I2S_BufferB;
            I2S2_MicrphoneBuffer.u8I2SCurBuffer = I2S2_MicrphoneBuffer.u8I2S_BufferA;
            
        }else
        {/*BufferB transmit complete*/
            I2S2_MicrphoneBuffer.u8I2SBufAB = 0;       /*Next DMA use Buffer A*/
            DMA1_Channel4->CMBA = (uint32_t)I2S2_MicrphoneBuffer.u8I2S_BufferA;
            I2S2_MicrphoneBuffer.u8I2SCurBuffer = I2S2_MicrphoneBuffer.u8I2S_BufferB;
            
        }
        DMA1_Channel4->TCNT = i2s_conf.u32MicrphoneBufLen / 2;
        DMA_ChannelEnable(DMA1_Channel4,TRUE);
    }
}
#endif
